; MShell V1.0 for TSE
; By Robin Kay
;

; You may not distribute a modified version of this program. However, you
; may take code from or base your program on this code providing that the
; resulting program is at least 50% of your own work.

; Some ideas for shell features:-
;  Modules
;  Folders
;  Cyclic Task Switching
;  ION Emulation
;  Display free memory/program size
;  Power down from within shell

.nolist
#include "tse.inc"
.list

.org    userMem-3
.db     $BB,$6D
 ret

 .db "TSE",1           ; Header
title:                 ; Title
 .db " MShell V1.0 - By Robin Kay", 0
 .dw 55                ; Stack Required
MenuTitle        .equ title + 1

start:

 ; Initalise Index and Selected
 xor a
 ld (MenuIndex), a
 ld (MenuSelect), a

rebuildMenu:
 ; Clear scren and display title
 bcall(_clrScrnFull)  
 xor a
 ld (pencol), a
 ld (penrow), a
 ld hl, MenuTitle
 bcall(_vputs)
 xor a
 ld (pencol), a
 ld a, (penrow)
 add a, 7
 ld (penrow), a
 ld bc, 0000h+(64-7)
 ld de, 5F00h+(64-7)
 ld h, 1
 bcall(_iline)
 ld c, 07h
 ld e, c
 bcall(_iline)

 ; Init Display menu
 xor a
 ld (MenuLimit), a
 ld a, (MenuIndex)
 ld b, a
loopDrawMenu:

 ; Get program title
 inc b
 push bc
 call getprog
 cp 0
 jr z, notFinished
 pop bc
 jr finished
notFinished:
 call _tseChkprog
 cp 0
 jr z, notFlash

 ; Mark if flash
Flash:
 push hl
 ld hl, arc
 bcall(_vputs)
 pop hl
notFlash:

 ; Display entry
 bcall(_vputs)
 xor a
 ld (pencol), a
 ld a, (penrow)
 add a, 6
 ld (penrow), a

 ; Increase limit
 ld hl, MenuLimit
 inc (hl)

 ; Check if menu full
 pop bc
 ld a, (hl)
 cp 8
 jr nz, loopDrawMenu
finished:

 ; Handle selected beyond end of menu
 ld a, (MenuSelect)
 ld b, a
 ld a, (MenuLimit)
 dec a
 cp b
 jr nc, skip1
 ld (MenuSelect), a
skip1:

 ; Handle empty menu
 ld a, (MenuLimit)
 cp 0
 jr nz, skip2
 ld a, (MenuIndex)
 cp 0
 jp z, _tseExitShell
 xor a
 ld (MenuIndex), a
 jp rebuildMenu
skip2:

 ; Highlight Entry
 ld a, (MenuSelect)
 call highlightMenuItem

 ; Implement delay
loopMenuKeypad:

 ; Poll keypad
 ei
 halt ; Save power
 bcall(_getCSC)

 ; Handle the quit key
 cp skDel
 call z, _tseExitShell

 ; Handle keys that refresh the display
 ld hl, rebuildMenu
 push hl
 cp skLeft
 jr z, moveLeft
 cp skRight
 jr z, moveRight
 cp skAlpha
 jr z, task_end
 cp sk2nd
 jr z, task_exec
 cp skMode
 jr z, arc_unarc
 pop hl

 ; Handle keys that't don't need to refresh display
 ld hl, loopMenuKeypad
 push hl
 cp skUp
 jr z, moveUp
 cp skDown
 jr z, moveDown

 ; Loop back up
 ret

 ; Move Left
moveLeft:
 ld a, (MenuIndex)
 add a, -8
 ld (MenuIndex), a 
 ret

 ; Move Right
moveRight:
 ld a, (MenuIndex)
 add a, 8
 ld (MenuIndex), a 
 ret

moveUp:

 ; Check if top of list
 ld a, (MenuSelect)
 cp 0
 ret z

 ; Move Up
 call highlightMenuItem
 dec a
 ld (MenuSelect), a
 call highlightMenuItem
 ret

moveDown:

 ; Check if bottom of list
 ld a, (MenuLimit)
 dec a
 ld c, a
 ld a, (MenuSelect)
 cp c
 ret z

 ; Move down
 call highlightMenuItem
 inc a
 ld (MenuSelect), a
 call highlightMenuItem
 ret

 ; Execute task
task_exec:
 call getsprog
 call _tseChkProg
 cp 0
 ret nz
 call _tseStartTask
 cp 4
 jr z, ok2
 cp 0
 ret nz
ok2:
 call _tseSwitchTask
 ret

 ; End task
task_end:
 call getsprog
 call _tseChkProg
 cp 0
 ret nz
 call _tseEndTask
 ret

 ; Archive/Unarchive program
arc_unarc:
 call getsprog
 call _tseChkProg

 ; Borrow system stack
 ld (tempword), sp
 ld hl, (sysstack)
 ld sp, hl

 ; Do arc/unarc
 bcall(_Arc_UnArc)

 ; Restore stack
 ld hl, (tempword)
 ld sp, hl
 ret

;--Functions--

; Highlight Menu Entry
; a should contain the menu item to be selected
;
highlightMenuItem:
 push af
 ld b, a
 ld a, 7
highlightLoop:
 add a, 6
 djnz highlightLoop  
 ld h, a
 add a, 6
 ld d, a
 ld e, 5Fh
 ld l, 0
 bcall(_invertRect)
 pop af
 ret

; Get number of selected program and runs into getprog
;
getsprog:
 ld a, (MenuSelect)
 ld b, a
 ld a, (MenuIndex)
 add a, b
 ld b, a
 inc b

; Convert TSE prog number into _tseFindVar number
; 
getprog:
 ld c, 0
getprogloop:
 inc c
 ld a, c
 push bc
 call _tseFindVar
 pop bc
 ld a, 0ffh
 ret nz
 push bc
 call _tseChkProg
 pop bc
 cp 2
 jr z, okFlash
 cp 0
 jr nz, getprogloop
okFlash:
 djnz getprogloop
 xor a
 ret

;--Data
arc:
.db "(A)",0

extdata:

MenuIndex    .equ extdata
MenuLimit    .equ MenuIndex+1
MenuSelect   .equ MenuLimit+1
tempword     .equ MenuSelect+1

.end

